# early linearization (using dco::jacobian_preaccumulator_t)

## Preprocessing

* `#include "dco_type.h"`
* globally search and replace `double` with `dco_type<double>`
* heston_mc.cpp, payoff.cpp: `using namespace std;` to facilitate overloading of `std::max` outside of standard namespace

## Driver

* main.cpp: 
  *  ...
  * for convenience: `std::vector<dco_type<double>*> X={&S_0,&K,&r,&v_0,&T,&rho,&kappa,&theta,&xi};`
  * `dco_mode<double>::global_tape=dco_tape_type<double>::create();`
  * `for (const auto& x:X) dco_mode<double>::global_tape->register_variable(*x);`
  *  ...
  * `dco_mode<double>::jacobian_preaccumulator_t jp(dco_mode<double>::global_tape);`
  * `for (unsigned i=0; i<num_sims; i++) {`
    * `jp.start();`
    *  ...
    * `jp.register_output(payoff_sum);`
    * `jp.finish();`
  * `}`
  *  ...
  * `dco_mode<double>::global_tape->register_output_variable(option_price);`
  * `dco::derivative(option_price)=1;`
  * `dco_mode<double>::global_tape->interpret_adjoint();`
  * `for (const auto& x:X) std::cout << dco::derivative(*x) << ' ';`
  * `dco_tape_type<double>::remove(dco_mode<double>::global_tape);`

## Result

* `Option Price: 6.75932`
* `d Option Price / d X: `
* `0.691955 -0.624362 62.367 21.1417 -nan -0.500163 0.0568204 109.002 0.0296506` 
* compile with `-DDCO_SQRT_AVOID_DIVZERO` eliminate `-nan`
