#ifndef __PAY_OFF_HPP
#define __PAY_OFF_HPP

#include "dco_type.h"

#include <algorithm> // This is needed for the std::max comparison function, used in the pay-off calculations

class PayOff {
 public:
  PayOff(); // Default (no parameter) constructor
  virtual ~PayOff() {}; // Virtual destructor

  // Overloaded () operator, turns the PayOff into an abstract function object
  virtual dco_type<double> operator() (const dco_type<double>& S) const = 0;
};

class PayOffCall : public PayOff {
 private:
  dco_type<double> K; // Strike price

 public:
  PayOffCall(const dco_type<double>& K_);
  virtual ~PayOffCall() {};

  // Virtual function is now over-ridden (not pure-virtual anymore)
  virtual dco_type<double> operator() (const dco_type<double>& S) const;
};

class PayOffPut : public PayOff {
 private:
  dco_type<double> K; // Strike

 public:
  PayOffPut(const dco_type<double>& K_);
  virtual ~PayOffPut() {};
  virtual dco_type<double> operator() (const dco_type<double>& S) const;
};

#endif
