#include "f12.h"
#include "dco.hpp"
#include <random>

template <typename T, typename Tp, typename To=float>
T f(int np, int nb, int ns, int nc, T x, const std::vector<std::vector<Tp>>& p) {
  using namespace std;
  typename dco::mode<T>::jacobian_preaccumulator_t jp(dco::tape(x));
  T y=0, x_in=x; To dt=1./ns;
  default_random_engine g; normal_distribution<To> d(0,1);
  for (int ib=0;ib<np;ib+=np/nb)
    for (int i=ib;i<ib+np/nb;++i) {
      x=x_in;
      for (int jc=0;jc<ns;jc+=ns/nc) {
        jp.start();
        for (int j=jc;j<jc+ns/nc;++j) 
          x+=dt*f1(x,p[0][j])+f2(x,p[1][j])*sqrt(dt)*d(g);
        jp.register_output(x);
        jp.finish();
      }
    y+=x;
  }
  return pow(y/np,2);
}

int main(int argc, char* argv[]) {
  assert(argc==5);
  int np=std::stoi(argv[1]), nb=std::stoi(argv[2]); 
  assert(np>0&&nb>0&&np%nb==0);
  int ns=std::stoi(argv[3]), nc=std::stoi(argv[4]); 
  assert(ns>0&&nc>0&&ns%nc==0);
  using Tb=double; std::cout.precision(std::numeric_limits<Tb>::digits10); 
  using Tp=dco::ga1s<Tb>::type; using T=Tp;
  T x=1.; std::vector<std::vector<Tp>> p(2,std::vector<Tp>(ns,1));
  dco::tape_options o;
  o.set_blob_size_in_gbyte(6);
  dco::ga1s<Tb>::global_tape=dco::ga1s<Tb>::tape_t::create(o); 
  dco::ga1s<Tb>::global_tape->register_variable(x);
  for (int i=0;i<ns;i++) { 
    dco::ga1s<Tb>::global_tape->register_variable(p[0][i]); 
    dco::ga1s<Tb>::global_tape->register_variable(p[1][i]); 
  }
  T y=f(np,nb,ns,nc,x,p); 
  dco::ga1s<Tb>::global_tape->register_output_variable(y); 
  dco::derivative(y)=1; 
  dco::ga1s<Tb>::global_tape->interpret_adjoint();
  std::cout << "dy/dx=" << dco::derivative(x) << '\n';
  for (int i=0;i<ns;i++) 
    std::cout << "dy/dp[0][" << i << "]=" << dco::derivative(p[0][i]) << '\n'
              << "dy/dp[1][" << i << "]=" << dco::derivative(p[1][i]) << std::endl;
  dco::ga1s<Tb>::tape_t::remove(dco::ga1s<Tb>::global_tape); 
  std::cout << "y=" << dco::value(y) << std::endl;
  return 0;
}

