#include "f.h" 
#include <cassert>
#include <limits>
#include <iostream>

template <typename T, typename Tp, typename To=float>
std::vector<T> df(int np, int ns, T x, std::vector<std::vector<Tp>> p) {
  int n=1+p.size()*p[0].size(); 
  std::vector<T> g(n);
  T eps=std::numeric_limits<T>::epsilon();
  T h=std::sqrt(std::sqrt(eps));
  h= (x!=0) ? h*std::fabs(x) : h;
  T yp=f(np,ns,x+h,p); T ym=f(np,ns,x-h,p); g[0]=(yp-ym)/(2*h); 
  for (int i=0,k=1;i<ns;++i) {
    Tp eps=std::numeric_limits<Tp>::epsilon();
    h=std::sqrt(std::sqrt(eps));
    h= (p[0][i]!=0) ? h*std::fabs(p[0][i]) : h;
    Tp aux=p[0][i]; 
    p[0][i]+=h; yp=f(np,ns,x,p); p[0][i]=aux;
    p[0][i]-=h; ym=f(np,ns,x,p); p[0][i]=aux;
    g[k++]=(yp-ym)/(2*h);
    h=std::sqrt(std::sqrt(eps));
    h= (p[1][i]!=0) ? h*std::fabs(p[1][i]) : h;
    aux=p[1][i]; 
    p[1][i]+=h; yp=f(np,ns,x,p); p[1][i]=aux;
    p[1][i]-=h; ym=f(np,ns,x,p); p[1][i]=aux;
    g[k++]=(yp-ym)/(2*h);
  }
  return g;
}

template <typename T, typename Tp, typename To=float>
std::vector<std::vector<T>> ddf(int np, int ns, T x, std::vector<std::vector<Tp>> p) {
  int n=1+p.size()*p[0].size(); 
  std::vector<std::vector<T>> H(n,std::vector<T>(n));
  T eps=std::numeric_limits<T>::epsilon();
  T h=std::sqrt(std::sqrt(eps));
  h= (x!=0) ? h*std::fabs(x) : h;
  std::vector<T> gp=df(np,ns,x+h,p); 
  std::vector<T> gm=df(np,ns,x-h,p); 
  for (int j=0;j<n;++j) H[0][j]=(gp[j]-gm[j])/(2*h); 
  for (int i=0,k=1;i<ns;++i) {
    Tp eps=std::numeric_limits<Tp>::epsilon();
    h=std::sqrt(std::sqrt(eps));
    h= (p[0][i]!=0) ? h*std::fabs(p[0][i]) : h;
    Tp aux=p[0][i]; 
    p[0][i]+=h; gp=df(np,ns,x,p); p[0][i]=aux;
    p[0][i]-=h; gm=df(np,ns,x,p); p[0][i]=aux;
    for (int j=0;j<n;++j) H[k][j]=(gp[j]-gm[j])/(2*h); 
    k++;
    h=std::sqrt(std::sqrt(eps));
    h= (p[1][i]!=0) ? h*std::fabs(p[1][i]) : h;
    aux=p[1][i]; 
    p[1][i]+=h; gp=df(np,ns,x,p); p[1][i]=aux;
    p[1][i]-=h; gm=df(np,ns,x,p); p[1][i]=aux;
    for (int j=0;j<n;++j) H[k][j]=(gp[j]-gm[j])/(2*h); 
    k++;
  }
  return H;
}

int main(int argc, char* argv[]) {
  assert(argc==3); 
  int np=std::stoi(argv[1]), ns=std::stoi(argv[2]); assert(np>0&&ns>0);
  using T=double; using Tp=float; 
  std::cout.precision(std::numeric_limits<T>::digits10);
  T x=1.; std::vector<std::vector<Tp>> p(2,std::vector<Tp>(ns,1));
  // primal
  std::cout << "y=" << f(np,ns,x,p) << std::endl;
  // Hessian
  std::vector<std::vector<T>> H=ddf(np,ns,x,p); 
  std::cout.precision(std::numeric_limits<T>::digits10/4);
  std::cout << "H=" << std::endl;
  for (int i=0;i<2*ns+1;++i) {
    for (int j=0;j<2*ns+1;++j) 
      std::cout << H[i][j] << ' ';
    std::cout << std::endl; 
  }
  return 0;
}
