#include "f.h"
#include "dco.hpp" 

int main(int argc, char* argv[]) {
  assert(argc==3); 
  int np=std::stoi(argv[1]), ns=std::stoi(argv[2]); assert(np>0&&ns>0);
  using Tbv=double; // base value type
  using Tbp=float; // base partial derivative type
  using Tba=float; // base adjoint type
  std::cout.precision(std::numeric_limits<Tba>::digits10); 
  using Tp=dco::ga1s<Tbv,Tbp,Tba>::type; using T=Tp;
  T x=1.; std::vector<std::vector<Tp>> p(2,std::vector<Tp>(ns,1));
  dco::ga1s<Tbv,Tbp,Tba>::global_tape=dco::ga1s<Tbv,Tbp,Tba>::tape_t::create(); 
  dco::ga1s<Tbv,Tbp,Tba>::global_tape->register_variable(x);
  for (int i=0;i<ns;i++) { 
    dco::ga1s<Tbv,Tbp,Tba>::global_tape->register_variable(p[0][i]); 
    dco::ga1s<Tbv,Tbp,Tba>::global_tape->register_variable(p[1][i]); 
  }
  T y=f(np,ns,x,p); 
  dco::ga1s<Tbv,Tbp,Tba>::global_tape->register_output_variable(y); 
  dco::derivative(y)=1; 
  dco::ga1s<Tbv,Tbp,Tba>::global_tape->interpret_adjoint(); 
  std::cout << "dy/dx=" << dco::derivative(x) << '\n';
  for (int i=0;i<ns;i++) 
    std::cout << "dy/dp[0][" << i << "]=" << dco::derivative(p[0][i]) << '\n'
              << "dy/dp[1][" << i << "]=" << dco::derivative(p[1][i]) << std::endl;
  dco::ga1s<Tbv,Tbp,Tba>::tape_t::remove(dco::ga1s<Tbv,Tbp,Tba>::global_tape); 
  std::cout << "y=" << dco::value(y) << std::endl;
  return 0;
}
