#include "f.h"
#include "dco.hpp"

int main(int argc, char* argv[]) {
  assert(argc==3); 
  int np=std::stoi(argv[1]), ns=std::stoi(argv[2]); assert(np>0&&ns>0);
  using Tbb=double; std::cout.precision(std::numeric_limits<Tbb>::digits10);
  using Tb=dco::gt1s<Tbb>::type; using Tp=dco::ga1s<Tb>::type; using T=Tp;
  T x=1,y; std::vector<std::vector<Tp>> p(2,std::vector<Tp>(ns,1));
  dco::ga1s<Tb>::global_tape=dco::ga1s<Tb>::tape_t::create(); 
  dco::ga1s<Tb>::global_tape->register_variable(x); 
  for (int i=0;i<ns;i++) { 
    dco::ga1s<Tb>::global_tape->register_variable(p[0][i]); 
    dco::ga1s<Tb>::global_tape->register_variable(p[1][i]); 
  }
  dco::ga1s<Tb>::tape_t::position_t pos=dco::ga1s<Tb>::global_tape->get_position();
  // d(dy/d(x,p))dx
  dco::derivative(dco::value(x))=1; 
  y=f(np,ns,x,p); 
  dco::value(dco::derivative(y))=1; 
  dco::ga1s<Tb>::global_tape->interpret_adjoint_and_reset_to(pos);
  std::cout << "d(dy/dx)/dx=" << dco::derivative(dco::derivative(x)) << std::endl;
  for (int j=0;j<ns;++j) {
    std::cout << "d(dy/dp[0][" << j << "])/dx=" << dco::derivative(dco::derivative(p[0][j])) << std::endl;
    dco::derivative(dco::value(p[0][j]))=0; dco::derivative(p[0][j])=0;
    std::cout << "d(dy/dp[1][" << j << "])/dx=" << dco::derivative(dco::derivative(p[1][j])) << std::endl;
    dco::derivative(dco::value(p[1][j]))=0; dco::derivative(p[1][j])=0;
  }
  dco::derivative(dco::value(x))=0; dco::derivative(x)=0; 
  // d(dy/d(x,p))dp
  for (int i=0;i<ns;++i) {
    dco::derivative(dco::value(p[0][i]))=1;
    y=f(np,ns,x,p); 
    dco::value(dco::derivative(y))=1; 
    dco::ga1s<Tb>::global_tape->interpret_adjoint_and_reset_to(pos);
    std::cout << "d(dy/dp[0][" << i << "])/dx=" << dco::derivative(dco::derivative(x)) << std::endl;
    dco::derivative(dco::value(x))=0; dco::derivative(x)=0; 
    for (int j=0;j<ns;++j) {
      std::cout << "d(dy/dp[0][" << i << "])/dp[0][" << j << "]=" << dco::derivative(dco::derivative(p[0][j])) << std::endl;
      dco::derivative(dco::value(p[0][j]))=0; dco::derivative(p[0][j])=0;
      std::cout << "d(dy/dp[0][" << i << "])/dp[1][" << j << "]=" << dco::derivative(dco::derivative(p[1][j])) << std::endl;
      dco::derivative(dco::value(p[1][j]))=0; dco::derivative(p[1][j])=0;
    }
    dco::derivative(dco::value(p[0][i]))=0;
    dco::derivative(dco::value(p[1][i]))=1;
    y=f(np,ns,x,p); 
    dco::value(dco::derivative(y))=1; 
    dco::ga1s<Tb>::global_tape->interpret_adjoint_and_reset_to(pos);
    std::cout << "d(dy/dp[1][" << i << "])/dx=" << dco::derivative(dco::derivative(x)) << std::endl;
    dco::derivative(dco::value(x))=0; dco::derivative(x)=0; 
    for (int j=0;j<ns;++j) {
      std::cout << "d(dy/dp[1][" << i << "])/dp[0][" << j << "]=" << dco::derivative(dco::derivative(p[0][j])) << std::endl;
      dco::derivative(dco::value(p[0][j]))=0; dco::derivative(p[0][j])=0;
      std::cout << "d(dy/dp[1][" << i << "])/dp[1][" << j << "]=" << dco::derivative(dco::derivative(p[1][j])) << std::endl;
      dco::derivative(dco::value(p[1][j]))=0; dco::derivative(p[1][j])=0;
    }
    dco::derivative(dco::value(p[1][i]))=0;
  }
  std::cout << "y=" << dco::value(dco::value(y)) << std::endl;
  return 0;
}
