#include "f.h"
#include "dco.hpp"

template<typename Tb, typename Tp>
Tb minimize(int np, int ns, Tb eps, Tb x_v, const std::vector<std::vector<Tp>>& p) {
  using namespace std;
  cout.precision(numeric_limits<Tb>::digits10); 
  using T=typename dco::ga1sm<Tb>::type; 
  T x=x_v,y;
  typename dco::ga1sm<Tb>::tape_t* tape=dco::ga1sm<Tb>::tape_t::create(); 
  tape->register_variable(x); 
  typename dco::ga1sm<Tb>::tape_t::position_t pos=tape->get_position();
  do {
    tape->zero_adjoints();
    y=f(np,ns,x,p); 
    tape->register_output_variable(y); 
    dco::derivative(y)=1;
    tape->interpret_adjoint_and_reset_to(pos);
    dco::value(x)-=0.1*dco::derivative(x);
    cout << "|dy/dx|=" << fabs(dco::derivative(x)) << endl;
    cout << "x=" << dco::value(x) << endl;
    cout << "y=" << dco::value(y) << endl;
  }
  while (fabs(dco::derivative(x))>eps);
  dco::ga1sm<Tb>::tape_t::remove(tape); 
  return dco::value(x);
}

int main(int argc, char* argv[]) {
  using namespace std;
  assert(argc==4); 
  int np=stoi(argv[1]), ns=stoi(argv[2]); assert(np>0&&ns>0);
  using Tb=double; 
  Tb eps=std::atof(argv[3]); assert(eps>0);
  using Tp=Tb;
  cout.precision(numeric_limits<Tb>::digits10); 
  Tb x=1.; vector<vector<Tp>> p(2,vector<Tp>(ns,1));
  x=minimize(np,ns,eps,x,p);
  return 0;
}
