#include "f.h"
#include "dco.hpp"

template<typename Tb, typename Tp>
Tb minimize(int np, int ns, const Tb& eps, Tb x_v, const std::vector<std::vector<Tp>>& p) {
  using namespace std;
  using T=typename dco::ga1sm<Tb>::type; 
  T x=x_v,y;
  typename dco::ga1sm<Tb>::tape_t* tape=dco::ga1sm<Tb>::tape_t::create(); 
  tape->register_variable(x); 
  typename dco::ga1sm<Tb>::tape_t::position_t pos=tape->get_position();
  do {
    tape->zero_adjoints();
    y=f(np,ns,x,p); 
    tape->register_output_variable(y); 
    dco::derivative(y)=1;
    tape->interpret_adjoint_and_reset_to(pos);
/*
    Tb y_ls=dco::value(y), alpha=1;
    while (y_ls>=y&&alpha>1e-3) {
      alpha/=2.;
      Tb x_ls=dco::value(x)-alpha*dco::derivative(x);
      y_ls=f(np,ns,x_ls,p); 
    }
    dco::value(x)-=alpha*dco::derivative(x);
*/
    dco::value(x)-=0.1*dco::derivative(x);
  }
  while (fabs(dco::derivative(x))>eps);
  dco::ga1sm<Tb>::tape_t::remove(tape); 
  return dco::value(x);
}

int main(int argc, char* argv[]) {
  assert(argc==4); 
  int np=std::stoi(argv[1]), ns=std::stoi(argv[2]); assert(np>0&&ns>0);
  using Tbb=double; 
  Tbb eps=std::stof(argv[3]); assert(eps>0);
  std::cout.precision(std::numeric_limits<Tbb>::digits10); 
  Tbb x_v=1.; std::vector<std::vector<Tbb>> p_v(2,std::vector<Tbb>(ns,1));
  x_v=minimize(np,ns,eps,x_v,p_v); 
  using Tb=dco::ga1sm<Tbb>::type; using T=dco::gt1s<Tb>::type; using Tp=Tb;
  T x=x_v; std::vector<std::vector<Tp>> p(2,std::vector<Tp>(ns,1));
  dco::derivative(x)=1; 
  dco::ga1sm<Tbb>::tape_t *tape=dco::ga1sm<Tbb>::tape_t::create(); 
  tape->register_variable(dco::value(x)); 
  for (int i=0;i<ns;i++) { 
    tape->register_variable(p[0][i]); 
    tape->register_variable(p[1][i]); 
  }
  T y=f(np,ns,x,p); 
  tape->register_output_variable(dco::derivative(y)); 
  dco::derivative(dco::derivative(y))=1; 
  tape->interpret_adjoint(); 
  for (int i=0;i<ns;i++) 
    std::cout << "dx/dp[0][" << i << "]=" << -dco::derivative(p[0][i])/dco::derivative(dco::value(x)) << '\n'
              << "dx/dp[1][" << i << "]=" << -dco::derivative(p[1][i])/dco::derivative(dco::value(x)) << std::endl;
  std::cout << "x=" << x_v << std::endl;
  dco::ga1sm<Tbb>::tape_t::remove(tape); 
  return 0;
}
