// Uwe.Naumann@nag.com
#pragma once
#include <cmath>

template<typename T>
T h(const T& x) {
  return pow(x,2);
}

template<typename T>
T g(const T& x, const T& h) {
  T v=x*h;
  return sin(v);
}

template<typename T, typename Tg>
T f(const T& x, const T& h, const Tg& g) {
  return x+h+g;
}

template<typename T, typename Tg>
void af(const T& x, T& ax, const T& h, T& ah, const Tg& g, T& ag, const T& ay) {
  ax=ah=ag=ay; ay=0;
}

template<typename T>
T F(const T& x) {
  T h_=h(x);
  return f(x,h_,g(x,h_));
}

#include "dco.hpp"

using TT=dco::gt1v<double,2>::type;
using AM=dco::ga1s<double>;
using AT=AM::type;

void inspect(const std::string& s) {
  using namespace std;
  cout << s << ':' << endl;
  cout << "MEM=" << dco::size_of(AM::global_tape) << endl;
  cout << "Tape: " << endl;
  dco::internal::algorithms::print(AM::global_tape);
  if (AM::global_tape->adjoints().size()>1) {
    cout << "Adjoints: " << endl;
    for (int i=0;i<AM::global_tape->adjoints().size();i++)
      cout << AM::global_tape->adjoints()[i] << endl;
  }
}

