#include "dco.hpp"
#include <iostream>
using namespace std;

template<typename T>
T h(const T& x) {
  return pow(x,2);
}

template<typename T>
T g(const T& x, const T& h) {
  T v=x*h;
  return sin(v);
}

template<typename T, typename Tg>
T f(const T& x, const T& h, const Tg& g) {
  return x+h+g;
}

template <typename T>
void ag(typename dco::ga1s<T>::external_adjoint_object_t *EA) {
  using AM=typename dco::ga1s<T>; using A=typename AM::type; 
  A x=EA->template read_data<T>(); 
  A h=EA->template read_data<T>(); 
  AM::global_tape->register_variable(x);
  AM::global_tape->register_variable(h);
  typename AM::tape_t::position_t pos=AM::global_tape->get_position();
  A gv=g(x,h); 
  dco::derivative(gv)=EA->get_output_adjoint();
  AM::global_tape->interpret_adjoint_and_reset_to(pos);
  EA->increment_input_adjoint(dco::derivative(x));
  EA->increment_input_adjoint(dco::derivative(h));
}

template<typename T>
typename dco::ga1s<T>::type g(const typename dco::ga1s<T>::type &x, const typename dco::ga1s<T>::type &h) { 
  using AM=typename dco::ga1s<T>; using A=typename AM::type; 
  typename AM::external_adjoint_object_t* EA=AM::global_tape->template create_callback_object<typename AM::external_adjoint_object_t>();
  EA->register_input(x);
  EA->register_input(h);
  T xv=dco::value(x); EA->write_data(xv);
  T hv=dco::value(h); EA->write_data(hv);
  T yv=g(xv,hv); 
  A gv=EA->register_output(yv);
  AM::global_tape->template insert_callback<typename AM::external_adjoint_object_t>(ag<T>,EA);
  return gv;
}

int main() {
  using P=double; 
  using AM=dco::ga1s<P>; 
  using A=AM::type; 
  AM::global_tape=AM::tape_t::create(); 
  A x=1.5,hv,gv,y; 
  AM::global_tape->register_variable(x); 
  hv=h(x);
  gv=g<P>(x,hv); 
  y=f(x,hv,gv); 
  dco::derivative(y)=1; 
  AM::global_tape->interpret_adjoint(); 
  cout << dco::value(y) << " " 
       << dco::derivative(x) << endl; 
  AM::tape_t::remove(AM::global_tape); 
  return 0;
}
